package provide DS9 1.0

proc CircleDialog {frame id} {
    global marker

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	return
    }

    # Init common items

    InitMarkerDialog $frame $id
    PropMenuCircleDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id CoordCircleCB

    # circle menu items

    set marker($frame,$id,radiusformat) pixels
    RadiusFormatMenuCircleDialog $frame $mb $id CoordCircleCB
    $mb add cascade -label Radius -menu $mb.radiusformat

    # Init circle variables

    EditCircleCB $frame $id

    # circle specific callbacks

    $frame marker $id callback edit EditCircleCB $frame
    $frame marker $id callback delete DeleteCircleDialog $frame

    # Dialog

    CommonMarkerDialog $frame $id
    wm protocol $w WM_DELETE_WINDOW "CloseCircleDialog $frame $id"

    # Radius

    label $w.ref.radiusTitle -text "Radius"
    entry $w.ref.radius -textvariable marker($frame,$id,radius) \
	-width 13 -relief sunken -bd 2
    label $w.ref.radiusformat -textvariable marker($frame,$id,radiusformat) \
	-relief groove -bd 2 -width 9 -padx 4

    grid $w.ref.radiusTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.ref.radius -row 1 -column 1 -padx 4 -sticky w
    grid $w.ref.radiusformat -row 1 -column 3 -padx 4 -sticky w

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyCircleDialog $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseCircleDialog $frame $id"
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyCircleDialog $frame $id"
}

proc PropMenuCircleDialog {frame mb id} {
    global menu
    global marker

    menu $mb.properties -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.properties add checkbutton -label "Can Edit" \
	-variable marker($frame,$id,edit) \
	-command "PropertyMarkerDialog $frame $id edit"
    $mb.properties add checkbutton -label "Can Move" \
	-variable marker($frame,$id,move) \
	-command "PropertyMarkerDialog $frame $id move"
    $mb.properties add checkbutton -label "Can Delete" \
	-variable marker($frame,$id,delete) \
	-command "PropertyMarkerDialog $frame $id delete"
    $mb.properties add checkbutton -label "Include/Exclude" \
	-variable marker($frame,$id,include) \
	-command "PropertyMarkerDialog $frame $id include"
    $mb.properties add checkbutton -label "Fixed in Size" \
	-variable marker($frame,$id,fixed) \
	-command "PropertyMarkerDialog $frame $id fixed"
}

proc RadiusFormatMenuCircleDialog {frame mb id cb} {
    global menu
    global marker

    menu $mb.radiusformat -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.radiusformat add radiobutton -label "Pixels" \
	-variable marker($frame,$id,radiusformat) -value pixels \
	-command "$cb $frame $id"
    if {[$frame has wcs equatorial]} {
	$mb.radiusformat add radiobutton -label "Degrees" \
	    -variable marker($frame,$id,radiusformat) -value degrees \
	    -command "$cb $frame $id"
	$mb.radiusformat add radiobutton -label "ArcMin" \
	    -variable marker($frame,$id,radiusformat) -value arcmin \
	    -command "$cb $frame $id"
	$mb.radiusformat add radiobutton -label "ArcSec" \
	    -variable marker($frame,$id,radiusformat) -value arcsec \
	    -command "$cb $frame $id"
    }
}

proc ApplyCircleDialog {frame id} {
    EditCircleDialog $frame $id
    ApplyMarkerDialog $frame $id
}

proc CloseCircleDialog {frame id} {
    global marker

    unset marker($frame,$id,radius)

    $frame marker $id delete callback edit EditCircleCB
    $frame marker $id delete callback delete DeleteCircleDialog
    DeleteMarkerCBs $frame $id

    DeleteMarkerDialog $frame $id
}

proc DeleteCircleDialog {frame id} {
    global marker

    unset marker($frame,$id,radius)
    unset marker($frame,$id,radiusformat)

    set mb ".mb$id"
    destroy $mb.radiusformat

    DeleteMarkerDialog $frame $id
}

proc EditCircleDialog {frame id} {
    global marker

    $frame marker $id circle radius $marker($frame,$id,radius) \
	$marker($frame,$id,coord) $marker($frame,$id,radiusformat)
}

proc EditCircleCB {frame id} {
    global marker

    set marker($frame,$id,radius) \
	[$frame get marker $id circle radius $marker($frame,$id,coord) \
	     $marker($frame,$id,radiusformat)]
}

proc CoordCircleCB {frame id} {
    MoveMarkerCB $frame $id
    EditCircleCB $frame $id
}
