package provide DS9 1.0

proc ContourColor {} {
    global contour
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) contour color $contour(color)
	UnsetWatchCursor
    }
}

proc ContourCreate {} {
    global contour
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) contour create $contour(color) \
	    $contour(levels) $contour(smooth)
	UnsetWatchCursor

	UpdateContourMenu
    }
}

proc ContourCopy {} {
    global contour
    global current

    set contour(copy) $current(frame)

    UpdateContourMenu
}

proc ContourPaste {sys} {
    global contour
    global current

    if {$current(frame) != "" && $contour(copy) != ""} {
	SetWatchCursor
	set pointer [$contour(copy) contour copy $sys]
	if {$pointer != 0} {
	    $current(frame) contour paste $contour(color) $pointer $sys
	}
	UnsetWatchCursor

	UpdateContourMenu
    }
}

proc ContourClear {} {
    global contour
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) contour delete
	UnsetWatchCursor

	UpdateContourMenu
    }
}

proc ContourDialog {} {
    global contour
    global current
    global menu
    global ctr
    global ds9

    # see if we already have a ctr window visible

    if [winfo exist $contour(top)] {
	return
    }

    set ctr(smooth) $contour(smooth)
    set ctr(levels) $contour(levels)

    set w $contour(top)
    set title "Contours"

    # create the contour window

    toplevel $contour(top)
    wm title $contour(top) $title
    wm iconname $contour(top) $title
    wm group $contour(top) $ds9(top)
    wm protocol $contour(top) WM_DELETE_WINDOW DestroyContourDialog

    $contour(top) configure -menu $contour(mb)

    menu $contour(mb) -tearoff 0
    $contour(mb) add cascade -label File -menu $contour(mb).file
    $contour(mb) add cascade -label Edit -menu $contour(mb).edit
    $contour(mb) add cascade -label Color -menu $contour(mb).color

    menu $contour(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).file add command -label "Close" \
	-command DestroyContourDialog

    menu $contour(mb).edit -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).edit add command -label "Copy" -command ContourCopy
    $contour(mb).edit add cascade -label "Paste" -menu $contour(mb).edit.paste
    $contour(mb).edit add command -label "Clear" -command ContourClear

    menu $contour(mb).edit.paste -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).edit.paste add command -label "WCS" \
	-command {ContourPaste fk4}
    $contour(mb).edit.paste add command -label "Detector" \
	-command {ContourPaste detector}
    $contour(mb).edit.paste add command -label "Amplifier" \
	-command {ContourPaste amplifier}
    $contour(mb).edit.paste add command -label "Physical" \
	-command {ContourPaste physical}
    $contour(mb).edit.paste add command -label "Image" \
	-command {ContourPaste image}

    menu $contour(mb).color -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).color add radiobutton -label "Black" \
	-variable contour(color) -value black -command ContourColor
    $contour(mb).color add radiobutton -label "White" \
	-variable contour(color) -value white -command ContourColor
    $contour(mb).color add radiobutton -label "Red" \
	-variable contour(color) -value red -command ContourColor
    $contour(mb).color add radiobutton -label "Green" \
	-variable contour(color) -value green -command ContourColor
    $contour(mb).color add radiobutton -label "Blue" \
	-variable contour(color) -value blue -command ContourColor
    $contour(mb).color add radiobutton -label "Cyan" \
	-variable contour(color) -value cyan -command ContourColor
    $contour(mb).color add radiobutton -label "Magenta" \
	-variable contour(color) -value magenta -command ContourColor
    $contour(mb).color add radiobutton -label "Yellow" \
	-variable contour(color) -value yellow -command ContourColor

    set length 300

    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    scale $w.param.num -from 0 -to 50 -length $length \
	-variable ctr(levels) -orient horizontal -label "Contour Levels" \
	-tickinterval 10 -showvalue true -resolution 1
    scale $w.param.res -from 0 -to 64 -length $length \
	-variable ctr(smooth) -orient horizontal \
	-label "Contour Smoothness" \
	-tickinterval 8 -showvalue true -resolution 1
    pack $w.param.num $w.param.res -side top -ipadx 4 -ipady 4 -anchor w

    button $w.buttons.apply -text "Apply" -command ApplyContourDialog
    button $w.buttons.close -text "Close" -command CloseContourDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyContourDialog"
}

proc ApplyContourDialog {} {
    global ctr
    global contour

    set contour(smooth) $ctr(smooth)
    if {$contour(smooth) < 1} {
	set contour(smooth) 1
    }

    set contour(levels) $ctr(levels)
    if {$contour(levels) < 1} {
	set contour(levels) 1
    }

    ContourCreate
}

proc CloseContourDialog {} {
    DestroyContourDialog
}

proc DestroyContourDialog {} {
    global contour

    destroy $contour(top)
    destroy $contour(mb).file
    destroy $contour(mb).edit
    destroy $contour(mb).color
    destroy $contour(mb)
}

proc UpdateContourMenu {} {
    global contour
    global current

    if {[winfo exist $contour(top)] && $current(frame) != ""} {
	
	# don't change contour color unless there is a contour
	set c [$current(frame) get contour color]
	if {$c != ""} {
	    set contour(color) $c
	}

	if {[$current(frame) has contour]} {
	    $contour(mb).edit entryconfig Copy -state normal
	    $contour(mb).edit entryconfig Clear -state normal
	} else {
	    $contour(mb).edit entryconfig Copy -state disabled
	    $contour(mb).edit entryconfig Clear -state disabled
	}

	if {$contour(copy) != ""} {
	    if {[$contour(copy) has contour]} {
		$contour(mb).edit entryconfig Paste -state normal

		if {[$current(frame) has wcs] && [$contour(copy) has wcs]} {
		    $contour(mb).edit.paste entryconfig WCS -state normal
		} else {
		    $contour(mb).edit.paste entryconfig WCS -state disabled
		}
	    } else {
		$contour(mb).edit entryconfig Paste -state disabled
	    }
	    
	} else {
	    $contour(mb).edit entryconfig Paste -state disabled
	}
    }
}
