package provide DS9 1.0

proc CreateGraphs {} {
    global graph
    global graphHorz
    global graphVert
    global canvas

    # we need to manually set the plot bg color, i.e. use the default bg
    set plotColor [.ds9 cget -background]

    set graph(spacer) [frame .ds9.spacer \
			   -width $graph(marginX) -height $graph(marginY)]

    set graphHorz(name) [blt::graph .ds9.horz \
			     -width $canvas(width) -height $graph(size) \
			     -bd 0 -plotborderwidth 2 -plotrelief groove \
			     -title "" \
			     -plotpadx 0 -plotpady 0 \
			     -leftmargin 1 -topmargin 4 \
			     -rightmargin $graph(marginX) \
			     -bottommargin 8 -plotbackground $plotColor]

    # wee need to manually set the element foreground color, i.e. use graph fg
    set fgColor [$graphHorz(name) cget -foreground]

    # Horizontal Graph

    $graphHorz(name) element create line1 \
	-xdata graphHorzX -ydata graphHorzY -color $fgColor -symbol ""
    $graphHorz(name) legend configure -hide yes
    $graphHorz(name) xaxis configure -title "" -hide yes
    $graphHorz(name) x2axis configure -title "" -hide yes
    $graphHorz(name) yaxis configure -title "" -hide yes
    $graphHorz(name) y2axis configure -title "" -hide no -color $fgColor \
	-ticklength 4 -tickfont $graph(font)
    $graphHorz(name) grid configure -mapy y -dashes ""
    $graphHorz(name) crosshairs configure -color green
    $graphHorz(name) element configure line1 -hide yes

    bind $graphHorz(name) <Enter> [list EnterGraph $graphHorz(name) 1]
    bind $graphHorz(name) <Leave> [list LeaveGraph $graphHorz(name)]
    bind $graphHorz(name) <Button-1> [list TrackGraph $graphHorz(name) %x %y 1]
    bind $graphHorz(name) <B1-Motion> \
	[list TrackGraph $graphHorz(name) %x %y 1]
    bind $graphHorz(name) <Up> [list ArrowKeyGraph $graphHorz(name) 0 -1 1]
    bind $graphHorz(name) <Down> [list ArrowKeyGraph $graphHorz(name) 0 1 1]
    bind $graphHorz(name) <Left> [list ArrowKeyGraph $graphHorz(name) -1 0 1]
    bind $graphHorz(name) <Right> [list ArrowKeyGraph $graphHorz(name) 1 0 1]

    # Vertical Graph

    set graphVert(name) [blt::graph .ds9.vert \
			     -width $graph(size) -height $canvas(height) \
			     -bd 0 -plotborderwidth 2 -plotrelief groove \
			     -title "" -invertxy yes \
			     -plotpadx 0 -plotpady 0 \
			     -leftmargin 8 -topmargin 1 \
			     -bottommargin $graph(marginY) \
			     -rightmargin 4 -plotbackground $plotColor]

    $graphVert(name) element create line1 \
	-xdata graphVertX -ydata graphVertY -color $fgColor -symbol ""
    $graphVert(name) legend configure -hide yes
    $graphVert(name) xaxis configure -title "" -hide yes -descending 1
    $graphVert(name) x2axis configure -title "" -hide yes -descending 1
    $graphVert(name) yaxis configure -title "" -hide no \
	-descending yes -color $fgColor -ticklength 4 -tickfont $graph(font)
    $graphVert(name) y2axis configure -title "" -hide yes -descending yes
    $graphVert(name) grid configure -mapy y -dashes ""
    $graphVert(name) crosshairs configure -color green
    $graphVert(name) element configure line1 -hide yes

    bind $graphVert(name) <Enter> [list EnterGraph $graphVert(name) 0]
    bind $graphVert(name) <Leave> [list LeaveGraph $graphVert(name)]
    bind $graphVert(name) <Button-1> [list TrackGraph $graphVert(name) %x %y 0]
    bind $graphVert(name) <B1-Motion> \
	[list TrackGraph $graphVert(name) %x %y 0]
    bind $graphVert(name) <Up> [list ArrowKeyGraph $graphVert(name) 0 -1 0]
    bind $graphVert(name) <Down> [list ArrowKeyGraph $graphVert(name) 0 1 0]
    bind $graphVert(name) <Left> [list ArrowKeyGraph $graphVert(name) -1 0 0]
    bind $graphVert(name) <Right> [list ArrowKeyGraph $graphVert(name) 1 0 0]

    UpdateGraphGrid
}

proc UpdateGraphGrid {} {
    global graphHorz
    global graphVert

    $graphHorz(name) grid configure -hide [expr !$graphHorz(grid)]
    $graphVert(name) grid configure -hide [expr !$graphVert(grid)]
}

proc UpdateGraphXAxis {} {
    global ds9
    global graphHorz
    global graphVert

    switch $ds9(displaymode) {
	single {
	    update idletasks
	    
	    if {$graphHorz(view)} {
		UpdateGraphXAxisHV $graphHorz(name) graphHorzX graphHorzY 1
	    }
	    
	    if {$graphVert(view)} {
		UpdateGraphXAxisHV $graphVert(name) graphVertX graphVertY 0
	    }
	}
	tile {}
	blink {}
    }
}

proc UpdateGraphXAxisHV {which vectorX vectorY cut} {
    global current
    global graph
    global graphHorzX graphHorzY
    global graphVertX graphVertY

    if {$current(frame) != ""} {

	if {$cut} {
	    $current(frame) get horizontal cut $vectorX $vectorY
	} else {
	    $current(frame) get vertical cut $vectorX $vectorY
	}

	# this is needed to deal with getting the min/max from a blt vector
	set foo "$$vectorX\(min\)"
	set bar "$$vectorX\(max\)"

	set xMin [expr $foo]
	set xMax [expr $bar]

	$which xaxis configure -min $xMin -max $xMax
	$which x2axis configure -min $xMin -max $xMax
    } else {
	$which xaxis configure -min $graph(xMin) -max $graph(xMax)
	$which x2axis configure -min $graph(xMin) -max $graph(xMax)
    }
}

proc UpdateGraphYAxis {} {
    global ds9
    global graphHorz
    global graphVert

    switch $ds9(displaymode) {
	single {
	    if {$graphHorz(view)} {
		UpdateGraphYAxisHV $graphHorz(name) $graphHorz(log)
	    }

	    if {$graphVert(view)} {
		UpdateGraphYAxisHV $graphVert(name) $graphVert(log)
	    }
	}
	tile {}
	blink {}
    }
}

proc UpdateGraphYAxisHV {which log} {
    global current
    global graph

    if {$current(frame) != ""} {

	set minmax [$current(frame) get clip]
	set yMin [lindex $minmax 0]
	set yMax [lindex $minmax 1]

	if {$yMin >= $yMax} {
	    set yMax [expr $yMin + 1]
	}

	$which yaxis configure -min $yMin -max $yMax -logscale $log
	$which y2axis configure -min $yMin -max $yMax -logscale $log
    } else {
	$which yaxis configure -min $graph(yMin) -max $graph(yMax) \
	    -logscale $log
	$which y2axis configure -min $graph(yMin) -max $graph(yMax) \
	    -logscale $log
    }
}

proc UpdateGraphData {} {
    global ds9
    global graphHorz
    global graphVert

    switch $ds9(displaymode) {
	single {
	    if {$graphHorz(view)} {
		UpdateGraphDataHV $graphHorz(name)
	    }
	    if {$graphVert(view)} {
		UpdateGraphDataHV $graphVert(name)
	    }
	}
	tile {}
	blink {}
    }
}

proc UpdateGraphDataHV {which} {
    global current

    if {$current(frame) != ""} {
	if {[$current(frame) get fits count] > 0} {
	    $which element configure line1 -hide no
	} else {
	    $which element configure line1 -hide yes
	}
    } else {
	$which element configure line1 -hide yes
    }
}

proc ClearGraphData {} {
    global graphHorz
    global graphVert

    if {$graphHorz(view)} {
	$graphHorz(name) element configure line1 -hide yes
    }    

    if {$graphVert(view)} {
	$graphVert(name) element configure line1 -hide yes
    }
}

proc MotionGraphCrosshair {which} {
    global ds9
    global graphHorz
    global graphVert

    switch $ds9(displaymode) {
	single {
	    if {[$which get fits count] > 0} {
		if {$graphHorz(view)} {
		    $which get horizontal cut graphHorzX graphHorzY
		    $graphHorz(name) element configure line1 -hide no
		}
		
		if {$graphVert(view)} {
		    $which get vertical cut graphVertX graphVertY
		    $graphVert(name) element configure line1 -hide no
		}
	    }
	}
	tile {}
	blink {}
    }
}

proc MotionGraphCanvas {which x y} {
    global ds9
    global view
    global graphHorz
    global graphVert

    switch $ds9(displaymode) {
	single {
	    if {[$which get fits count] > 0} {
		if {$graphHorz(view)} {
		    $which get horizontal cut graphHorzX graphHorzY $x $y
		    $graphHorz(name) element configure line1 -hide no
		}

		if {$graphVert(view)} {
		    $which get vertical cut graphVertX graphVertY $x $y
		    $graphVert(name) element configure line1 -hide no
		}
	    }
	}
	tile {}
	blink {}
    }
}

proc EnterGraph {which horz} {
    global ds9
    global current

    focus $which
    $which crosshairs on

    if {$current(frame) != ""} {
	set cursor [$which crosshairs cget -position]
	scan $cursor "@%d,%d" x y

	if {$ds9(centerpixel)} {
	    if {$horz} {
		set x [expr $x+.5]
	    } else {
		set y [expr $y+.5]
	    }
	}

	set coord [$current(frame) get crosshair]
	set X [lindex $coord 0]
	set Y [lindex $coord 1]

	if {$horz} {
	    UpdateInfoBox $current(frame) $x $Y canvas
	} else {
	    UpdateInfoBox $current(frame) $X $y canvas
	}
    }
}

proc LeaveGraph {which} {
    focus {}
    $which crosshairs off
    ClearInfoBox
}

proc TrackGraph {which x y horz} {
    global ds9
    global current

    $which crosshairs configure -position "@$x,$y"

    if {$current(frame) != ""} {
	if {$ds9(centerpixel)} {
	    if {$horz} {
		set x [expr $x+.5]
	    } else {
		set y [expr $y+.5]
	    }
	}

	set coord [$current(frame) get crosshair]
	set X [lindex $coord 0]
	set Y [lindex $coord 1]

	if {$horz} {
	    UpdateInfoBox $current(frame) $x $Y canvas
	} else {
	    UpdateInfoBox $current(frame) $X $y canvas
	}
    }    
}

proc ArrowKeyGraph {which x y horz} {
    set cursor [$which crosshairs cget -position]
    scan $cursor "@%d,%d" cx cy

    set cx [expr $cx+$x]
    set cy [expr $cy+$y]

    TrackGraph $which $cx $cy $horz
}
