package provide DS9 1.0

proc GridCreate {} {
    global grid
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) grid create $grid(coordsystem) $grid(coordformat) \
	    $grid(color)
	UnsetWatchCursor

	UpdateGridMenu
    }
}

proc GridClear {} {
    global grid
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) grid delete
	UnsetWatchCursor

	UpdateGridMenu
    }
}

proc GridColor {} {
    global grid
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) grid color $grid(color)
	UnsetWatchCursor
    }
}

proc GridCoordFormat {} {
    global grid
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) grid format $grid(coordformat)
	UnsetWatchCursor
    }
}

proc GridDialog {} {
    global grid
    global current
    global menu
    global gr
    global ds9

    # see if we already have a grid window visible

    if [winfo exist $grid(top)] {
	return
    }

    if [$current(frame) has grid] {
	set grid(coordsys) [$current(grid) get grid system]
	set grid(coordformat) [$current(grid) get grid format]
	set grid(color) [$current(grid) get grid color]
    }

    set w $grid(top)
    set title "Coordinate Grid"

    # create the grid window

    toplevel $grid(top)
    wm title $grid(top) $title
    wm iconname $grid(top) $title
    wm group $grid(top) $ds9(top)
    wm protocol $grid(top) WM_DELETE_WINDOW DestroyGridDialog

    $grid(top) configure -menu $grid(mb)

    menu $grid(mb) -tearoff 0
    $grid(mb) add cascade -label File -menu $grid(mb).file
    $grid(mb) add cascade -label Edit -menu $grid(mb).edit
    $grid(mb) add cascade -label Coordinate -menu $grid(mb).coord
    $grid(mb) add cascade -label Color -menu $grid(mb).color

    menu $grid(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $grid(mb).file add command -label "Close" -command DestroyGridDialog

    menu $grid(mb).edit -tearoff 0 -selectcolor $menu(selectcolor)
    $grid(mb).edit add command -label "Copy" -state disabled
    $grid(mb).edit add command -label "Paste" -state disabled
    $grid(mb).edit add command -label "Clear" -command GridClear

    menu $grid(mb).coord -tearoff 0 -selectcolor $menu(selectcolor)
    $grid(mb).coord add radiobutton -label "Equatorial B1950" \
	-variable grid(coordsystem) -value fk4 -command GridCreate
    $grid(mb).coord add radiobutton -label "Equatorial J2000" \
	-variable grid(coordsystem) -value fk5 -command GridCreate
    $grid(mb).coord add radiobutton -label "ICRS" \
	-variable grid(coordsystem) -value icrs -command GridCreate
    $grid(mb).coord add radiobutton -label "Galactic" \
	-variable grid(coordsystem) -value galatic -command GridCreate
    $grid(mb).coord add radiobutton -label "Ecliptic" \
	-variable grid(coordsystem) -value ecliptic -command GridCreate
    $grid(mb).coord add radiobutton -label "Detector" \
	-variable grid(coordsystem) -value detector -command GridCreate
    $grid(mb).coord add radiobutton -label "Amplifier" \
	-variable grid(coordsystem) -value amplifier -command GridCreate
    $grid(mb).coord add radiobutton -label "Physical" \
	-variable grid(coordsystem) -value physical -command GridCreate
    $grid(mb).coord add radiobutton -label "Image" \
	-variable grid(coordsystem) -value image -command GridCreate
    $grid(mb).coord add separator
    $grid(mb).coord add radiobutton -label "Degrees" \
	-variable grid(coordformat) -value degrees -command GridCoordFormat
    $grid(mb).coord add radiobutton -label "HMS" \
	-variable grid(coordformat) -value hms -command GridCoordFormat

    menu $grid(mb).color -tearoff 0 -selectcolor $menu(selectcolor)
    $grid(mb).color add radiobutton -label "Black" \
	-variable grid(color) -value black -command GridColor
    $grid(mb).color add radiobutton -label "White" \
	-variable grid(color) -value white -command GridColor
    $grid(mb).color add radiobutton -label "Red" \
	-variable grid(color) -value red -command GridColor
    $grid(mb).color add radiobutton -label "Green" \
	-variable grid(color) -value green -command GridColor
    $grid(mb).color add radiobutton -label "Blue" \
	-variable grid(color) -value blue -command GridColor
    $grid(mb).color add radiobutton -label "Cyan" \
	-variable grid(color) -value cyan -command GridColor
    $grid(mb).color add radiobutton -label "Magenta" \
	-variable grid(color) -value magenta -command GridColor
    $grid(mb).color add radiobutton -label "Yellow" \
	-variable grid(color) -value yellow -command GridColor

    set length 300

    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    button $w.buttons.apply -text "Apply" -command ApplyGridDialog
    button $w.buttons.close -text "Close" -command DestroyGridDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyGridDialog"

    UpdateGridMenu
}

proc ApplyGridDialog {} {
    global grid
    global gr

    GridCreate
}

proc DestroyGridDialog {} {
    global grid

    destroy $grid(top)
    destroy $grid(mb).file
    destroy $grid(mb).edit
    destroy $grid(mb).coord
    destroy $grid(mb).color
    destroy $grid(mb)
}

proc UpdateGridMenu {} {
    global grid
    global current

    if {[winfo exist $grid(top)] && $current(frame) != ""} {
	
	# don't change grid color unless there is a grid
	set c [$current(frame) get grid color]
	if {$c != ""} {
	    set grid(color) $c
	}

	if {[$current(frame) has grid]} {
	    $grid(mb).edit entryconfig Clear -state normal
	} else {
	    $grid(mb).edit entryconfig Clear -state disabled
	}

	if {[$current(frame) has wcs equatorial]} {
	    $grid(mb).coord entryconfig "Equatorial B1950" -state normal
	    $grid(mb).coord entryconfig "Equatorial J2000" -state normal
	    $grid(mb).coord entryconfig "ICRS" -state normal
	    $grid(mb).coord entryconfig "Galactic" -state normal
	    $grid(mb).coord entryconfig "Ecliptic" -state normal
	} else {
	    $grid(mb).coord entryconfig "Equatorial B1950" -state disabled
	    $grid(mb).coord entryconfig "Equatorial J2000" -state disabled
	    $grid(mb).coord entryconfig "ICRS" -state disabled
	    $grid(mb).coord entryconfig "Galactic" -state disabled
	    $grid(mb).coord entryconfig "Ecliptic" -state disabled
	}

	if {[$current(frame) has mosaic] && \
	    [$current(frame) has physical] && 0} {
	    $grid(mb).coord entryconfig "Detector" -state normal
	} else {
	    $grid(mb).coord entryconfig "Detector" -state disabled
	}

	if {[$current(frame) has mosaic] && 0} {
	    $grid(mb).coord entryconfig "Amplifier" -state normal
	} else {
	    $grid(mb).coord entryconfig "Amplifier" -state disabled
	}

	if {[$current(frame) has physical] && 0} {
	    $grid(mb).coord entryconfig "Physical" -state normal
	} else {
	    $grid(mb).coord entryconfig "Physical" -state disabled
	}
    }
}
