#ifndef __fitshdu_h__
#define __fitshdu_h__

#include <stdlib.h>

#include "column.h"

#define FT_MAXAXES 7

class FitsHead;

static char* cat(const char*,int);

class FitsHDU {
protected:
  char* extname_;             // EXTNAME keyword
  int extver_;                // EXTVER keyword

  int bitpix_;                // BITPIX keyword
  int naxes_;                 // NAXIS keyword
  int naxis_[FT_MAXAXES];     // NAXIS[i] keywords

  int databytes_;             // Number of bytes in data segment (include padding)
  int datablocks_;            // Number of data blocks

public:
  FitsHDU(FitsHead*);
  virtual ~FitsHDU();

  virtual void updateCards(FitsHead*);

  const char* extname() {return extname_;}
  int extver() {return extver_;}
  int bitpix() {return bitpix_;}
  int naxes() {return naxes_;}
  int naxis(int i) {return naxis_[i-1];}
  int databytes() {return databytes_;}
  int datablocks() {return datablocks_;}
};

class FitsImageHDU : public FitsHDU {
private:
  int datapixels_;

  double bscale_;
  double bzero_;
  int hasblank_;
  int blank_;

public:
  FitsImageHDU(FitsHead*);

  void updateCards(FitsHead*);

  int datapixels() {return datapixels_;}
  double bscale() {return bscale_;}
  double bzero() {return bzero_;}
  int hasscaling() {return bscale_ != 1 || bzero_ != 0;}
  int hasblank() {return hasblank_;}
  int blank() {return blank_;}
};

class FitsTableHDU : public FitsHDU {
protected:
  int tfields_;
  FitsColumn** cols_;

public:
  FitsTableHDU(FitsHead*);
  virtual ~FitsTableHDU();

  int tfields() {return tfields_;}
  int rows() {return naxis_[1];}
  int cols() {return tfields_;}
  int width() {return naxis_[0];}
  FitsColumn* find(const char*);
  Vector dimension(const char*);
};

class FitsBinTableHDU : public FitsTableHDU {
private:

public:
  FitsBinTableHDU(FitsHead*);

};

class FitsAsciiTableHDU : public FitsTableHDU {
private:

public:
  FitsAsciiTableHDU(FitsHead*);

};

#endif


