%{
#define YYPARSE_PARAM ff
#define FF ((FitsFile*)ff)
#define YYDEBUG 1

#include "iostream.h"
#include "file.h"

extern int fflex(void);
extern void fferror(const char*);

%}

%union {
  float real;
  int integer;
  char str[256];
  void* ptr;
}

%type <real> numeric

%token <real> REAL
%token <integer> INT
%token <str> STRING

%token ARCH_
%token BIGENDIAN_
%token BIN_
%token BINKEY_
%token BITPIX_
%token DIM_
%token DIMS_
%token KEY_
%token LITTLEENDIAN_
%token SKIP_
%token XDIM_
%token YDIM_

%%

command	: filename
	| filename options
	| filename '[' extn ','  bin ']'
	| filename '[' arrs ']'
	;

numeric	: REAL {$$=$1;}
	| INT {$$=$1;}
	;

options	: options option
	| option
	;

option	: '[' extn ']'
	| '[' bin ']'
	;

filename : /* empty */
	|  STRING {FF->setpName($1);}
	;

extn	: STRING {FF->setpExt($1);}
	| INT {FF->setpIndex($1);}
	;

bin	: BINKEY_ '=' binkey
	| BINKEY_ binkey
	| KEY_ '=' binkey
	| KEY_ binkey
	| BIN_ '=' binkey
	| BIN_ binkey
	;

binkey	: '(' STRING ',' STRING ')' {FF->setpBinXY($2,$4);}
	| STRING ',' STRING {FF->setpBinXY($1,$3);}
	| '(' STRING ',' STRING ',' STRING ')' {FF->setpBinXYZ($2,$4,$6);}
	| STRING ',' STRING ',' STRING {FF->setpBinXYZ($1,$3,$5);}
	| '(' STRING ')' {FF->setpBinZ($2);}
	| STRING {FF->setpBinZ($1);}
	;

arrs	: arrs ',' arr
	| arr
	;

arr	: XDIM_ '=' INT {FF->setpXdim($3);}
	| YDIM_ '=' INT {FF->setpYdim($3);}
	| DIM_ '=' INT {FF->setpXdim($3);FF->setpYdim($3);}
	| DIMS_ '=' INT {FF->setpXdim($3);FF->setpYdim($3);}
	| BITPIX_ '=' INT {FF->setpBitpix($3);}
	| SKIP_ '=' INT {FF->setpSkip($3);}
	| ARCH_ '=' endian
	| endian
	;

endian	: BIGENDIAN_ {FF->setpArch(FitsFile::BIGENDIAN);}
	| LITTLEENDIAN_ {FF->setpArch(FitsFile::LITTLEENDIAN);}
	;

%%

