#include "framepseudo8.h"
#include "colorscalepseudo8.h"
#include "fitsimage.h"
#include "nan.h"
#include "ps.h"
#include "util.h"

// Tk Canvas Widget Function Declarations

int FramePseudoColor8CreateProc(Tcl_Interp*, Tk_Canvas, Tk_Item*, int, char**);

// FramePseudoColor8 Specs

static Tk_CustomOption tagsOption = {
  Tk_CanvasTagsParseProc, Tk_CanvasTagsPrintProc, NULL
};

static Tk_ConfigSpec framePseudoColor8Specs[] = {

  {TK_CONFIG_STRING, "-command", NULL, NULL, "frame",
   Tk_Offset(WidgetOptions, cmdName), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-x", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, x), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-y", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, y), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-width", NULL, NULL, "512",
   Tk_Offset(WidgetOptions, width), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-height", NULL, NULL, "512",
   Tk_Offset(WidgetOptions, height), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_ANCHOR, "-anchor", NULL, NULL, "nw",
   Tk_Offset(WidgetOptions, anchor), 0, NULL},
  {TK_CONFIG_CUSTOM, "-tags", NULL, NULL, NULL,
   0, TK_CONFIG_NULL_OK, &tagsOption},

  {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0, NULL},
};

// Tk Static Structure

static Tk_ItemType framePseudoColor8Type = {
  "framepseudocolor8",          // name
  sizeof(WidgetOptions),        // item size
  FramePseudoColor8CreateProc,  // configProc
  framePseudoColor8Specs,       // configSpecs
  WidgetConfigProc,             // configProc
  WidgetCoordProc,              // coordProc
  WidgetDeleteProc,             // deleteProc
  WidgetDisplayProc,            // displayProc
  0,                            // alwaysRedraw
  WidgetPointProc,              // pointProc
  WidgetAreaProc,               // areaProc
  WidgetPostscriptProc,         // postscriptProc
  WidgetScaleProc,              // scaleProc
  WidgetTranslateProc,          // translateProc
  (Tk_ItemIndexProc*)NULL,      // indexProc
  WidgetICursorProc,            // icursorProc
  (Tk_ItemSelectionProc*)NULL,  // selectionProc
  (Tk_ItemInsertProc*)NULL,     // insertProc
  (Tk_ItemDCharsProc*)NULL,     // dCharsProc
  (Tk_ItemType*)NULL            // nextPtr
};

// Non-Member Functions

int FramePseudoColor8_Init(Tcl_Interp* interp)
{
  Tk_CreateItemType(&framePseudoColor8Type);
  return TCL_OK;
}

int FramePseudoColor8CreateProc(Tcl_Interp* interp, Tk_Canvas canvas, 
				Tk_Item* item, int argc, char** argv)
{
  FramePseudoColor8* frame = new FramePseudoColor8(interp, canvas, item);

  // and set default configuration

  if (frame->configure(argc, argv, 0) != TCL_OK) {
    delete frame;
    Tcl_AppendResult(interp, " error occured while creating frame.", NULL);
    return TCL_ERROR;
  }

  return TCL_OK;
}

// FramePseudoColor8 Member Functions

FramePseudoColor8::FramePseudoColor8(Tcl_Interp* i, Tk_Canvas c, Tk_Item* item)
  : Frame(i, c, item)
{
  configSpecs = framePseudoColor8Specs;  // frame configure options
}

FramePseudoColor8::~FramePseudoColor8()
{
}

void FramePseudoColor8::updateColorScale()
{
  if (DebugColorbar)
    cerr << "FramePseudo8::updateColorScale" << endl;

  // we need colors before we can construct a scale

  if (!indexCells || !colorCells)
    return;

  if (colorScale)
    delete colorScale;

  switch (colorScaleType) {
  case LINEARSCALE:
    colorScale =
      new LinearScalePseudoColor8(colorCount, indexCells, colorCells, 
				  colorCount);
    break;
  case LOGSCALE:
    colorScale = new LogScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
					  colorCount);
    break;
  case SQUAREDSCALE:
    colorScale =
      new SquaredScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
				   colorCount);
    break;
  case SQRTSCALE:
    colorScale = 
      new SqrtScalePseudoColor8(SCALESIZE, indexCells, colorCells, colorCount);
    break;
  case IISSCALE:
    colorScale =
      new LinearScalePseudoColor8(200, indexCells, colorCells, colorCount);
    break;
  }
}

void FramePseudoColor8::clearPixmap(Pixmap pmap, XImage* xmap)
{
  XFillRectangle(display, pmap, gc, 0, 0, options->width, options->height);
}

void FramePseudoColor8::fillXImage(XImage* xmap, 
				   Matrix& (FitsImage::*getMatrix)())
{
  // we need a colorScale before we can render

  if (!colorScale)
    return;

  // Xfer pixels from base image into map image--
  // This section of code has been optimized. First, the matix multiply
  // done inline, only using elements that count. (that is the first 6)
  // Second, the pixel xfer assumes 1 byte/pixel, and array indexing is
  // kepted to a minimum.

  int width = xmap->width;
  int height = xmap->height;

  // clear ximage

  memset(xmap->data, '\0', xmap->bytes_per_line * height);

  Matrix& m = (fits->*getMatrix)();
  register double m00 = m.matrix(0,0);
  register double m10 = m.matrix(1,0);
  register double m20 = m.matrix(2,0);
  register double m01 = m.matrix(0,1);
  register double m11 = m.matrix(1,1);
  register double m21 = m.matrix(2,1);
  register int srcWidth = fits->width();
  register int srcHeight = fits->height();

  register int length = colorScale->size() - 1;
  register const unsigned char* table = colorScale->colors();

  register double ll = fits->getLowDouble();
  register double hh = fits->getHighDouble();
  register double diff = hh - ll;
  int& bytesPerLine = xmap->bytes_per_line;

  // test as doubles, convert to int's at last minute.
  // use double to maintain precision
  // This solves a host of problems.

  for (int j=0; j<height; j++) {

    // the line may be padded at the end
    register char* dest = xmap->data + j*bytesPerLine;

    for (int i=0; i<width; i++,dest++) {

      register double srcX = i*m00 + j*m10 + m20;
      register double srcY = i*m01 + j*m11 + m21;

      if (srcX >= 0 && srcX < srcWidth && srcY >= 0 && srcY < srcHeight) {
	register double value = fits->getValueDouble(((int)srcY)*srcWidth + 
						     (int)srcX);
	
	if (!isnand(value))
	  if (value <= ll)
	    *dest = table[0];
	  else if (value >= hh)
	    *dest = table[length];
	  else
	    *dest = table[(int)(((value - ll)/diff * length) + .5)];
      }
    }
  }
}

void FramePseudoColor8::fillXImageMosaic(XImage* xmap, 
					 Matrix& (FitsImage::*getMatrix)())
{
  // we need a colorScale before we can render

  if (!colorScale)
    return;

  // Xfer pixels from base image into map image--
  // This section of code has been optimized. First, the matix multiply
  // done inline, only using elements that count. (that is the first 6)
  // Second, the pixel xfer assumes 1 byte/pixel, and array indexing is
  // kepted to a minimum.

  // We cycle thru each pixel on the widget. For each pixel, 
  // we cycle thru each fits until we've run out of fits or 
  // we have a hit. If we hit, we remember it and use it until we
  // miss again. If we do not find a hit, we go on to the next pixel.

  int width = xmap->width;
  int height = xmap->height;

  // clear ximage

  memset(xmap->data, '\0', xmap->bytes_per_line * height);

  register int length = colorScale->size() - 1;
  register const unsigned char* table = colorScale->colors();

  register double m00;
  register double m10;
  register double m20;
  register double m01;
  register double m11;
  register double m21;
  register int srcWidth;
  register int srcHeight;

  register double ll;
  register double hh;
  register double diff;
  int& bytesPerLine = xmap->bytes_per_line;

  // test as doubles, convert to int's at last minute.
  // use double to maintain precision
  // This solves a host of problems.

  int found = 0;
  FitsImage* current = fits;
  FitsImage* ptr;

  for (int j=0; j<height; j++) {

    // the line may be padded at the end
    register char* dest = xmap->data + j*bytesPerLine;

    for (int i=0; i<width; i++,dest++) {

    tryagain:
      if (!found) {
	ptr = current;
	current = current->next();

	Matrix& m = (ptr->*getMatrix)();
	m00 = m.matrix(0,0);
	m10 = m.matrix(1,0);
	m20 = m.matrix(2,0);
	m01 = m.matrix(0,1);
	m11 = m.matrix(1,1);
	m21 = m.matrix(2,1);
	srcWidth = ptr->width();
	srcHeight = ptr->height();

	ll = ptr->getLowDouble();
	hh = ptr->getHighDouble();
	diff = hh - ll;
      }

      register double srcX = i*m00 + j*m10 + m20;
      register double srcY = i*m01 + j*m11 + m21;

      if (srcX >= 0 && srcX < srcWidth && srcY >= 0 && srcY < srcHeight) {
	register double value = ptr->getValueDouble(((int)srcY)*srcWidth + 
						    (int)srcX);
	if (!isnand(value))
	  if (value <= ll)
	    *dest = table[0];
	  else if (value >= hh)
	    *dest = table[length];
	  else
	    *dest = table[(int)(((value - ll)/diff * length) + .5)];
	found = 1;
      }
      else {
	found = 0;
	if (current)
	  goto tryagain;
	else
	  current = fits;
      }
    }
  }
}

void FramePseudoColor8::updateMagnifierReplicate(const Vector& v)
{
  // v is in CANVAS Coords
  // we have a race condition here. It's possible that a magnifier command has
  // been generated right after the pixmap has been invalidated and before
  // a new XImage has been generated. So, bail, and try again.

  if (!baseXImage || !magnifierXImage)
    return;

  char* image = baseXImage->data;
  char* mag = magnifierXImage->data;

  // map upper left of magnifier to image

  Matrix m = Translate(-(v * canvasToWidget)) * ximageToMagnifier;
  Matrix n = m.invert();

  Vector ll = Vector(0,0) * n;

  // and init image row and column counters

  int imageY = (int)ll[1];

  // loop thru each magnifier pixel

  char* row = new char[magnifierWidth];
  for (int j=0; j<=magnifierHeight; j+=(int)magnifierZoom, imageY++) {

    // Clear tmp row

    memset(row, 0, magnifierWidth);

    // Build tmp row

    int imageX = (int)ll[0];
    for (int i=0; i<=magnifierWidth; i+=(int)magnifierZoom, imageX++) {

      // find pixel value

      unsigned char value;
      if (imageX>=0 && imageX<options->width && 
	  imageY>=0 && imageY<options->height)
	value = image[imageY*baseXImage->bytes_per_line + imageX];
      else
	value = 0;

      // replicate pixel

      int start = i - (int)(magnifierZoom/2.);
      if (start < 0)
	start = 0;
      int stop = i + (int)(magnifierZoom/2.);
      if (stop > magnifierWidth)
	stop = magnifierWidth;
      for (int k=start; k<stop; k++) {
	  row[k] = value;
      }
    }

    // and replicate temp row
    
    int start = j - (int)(magnifierZoom/2.);
    if (start < 0)
      start = 0;
    int stop = j + (int)(magnifierZoom/2.);
    if (stop > magnifierHeight)
      stop = magnifierHeight;
    for (int k=start; k<stop; k++)
	memcpy(mag+(k*magnifierXImage->bytes_per_line), row, magnifierWidth);
  }
  delete row;

  XPutImage(display, magnifierPixmap, gc, magnifierXImage, 0, 0, 0, 0, 
	    magnifierWidth, magnifierHeight);
}

void FramePseudoColor8::colormapCmd(int id, float b, float c, int i, 
				    unsigned short* index, 
				    unsigned char* cells, int cnt)
{
  if (DebugColorbar)
    cerr << "FramePseudoColor8::setColormapCmd" << endl;

  cmapID = id;
  bias = b;
  contrast = c;
  invert = i;

  updateColorCells(index, cells, cnt);
  updateColorScale();
}

void FramePseudoColor8::rotateMotion()
{
  // Rotate from src to dest

  Vector c = Vector(options->width,options->height)/2;
  Matrix m = Translate(-c) * Rotate(rotation-rotateRotation) * Translate(c);
  Matrix mm = m.invert();
  register int width = options->width;
  register int height = options->height;
  register double m00 = mm.matrix(0,0);
  register double m10 = mm.matrix(1,0);
  register double m20 = mm.matrix(2,0);
  register double m01 = mm.matrix(0,1);
  register double m11 = mm.matrix(1,1);
  register double m21 = mm.matrix(2,1);
  register char* src = rotateSrcXM->data;

  for (register int j=0; j<height; j++) {
    // the line may be padded at the end
    register char* dest = rotateDestXM->data + j*rotateDestXM->bytes_per_line;

    for (register int i=0; i<width; i++, dest++) {
      register double srcX = i*m00 + j*m10 + m20;
      register double srcY = i*m01 + j*m11 + m21;

      if (srcX >= 0 && srcX < width && srcY >= 0 && srcY < height)
	*dest = src[((int)srcY)*rotateSrcXM->bytes_per_line + (int)srcX];
      else
	*dest = 0;
    }
  }

  // XImage to Pixmap

  XPutImage(display, rotatePM, gc, rotateDestXM,
	    0, 0, 0, 0, options->width, options->height);

  // Display Pixmap

  Vector d = Vector(0,0) * widgetToWindow;
  XCopyArea(display, rotatePM, Tk_WindowId(tkwin), rotateGCXOR, 0, 0, 
	    options->width, options->height, (int)d[0], (int)d[1]);
}
