#include <iostream.h>
#include <strstream.h>
#include <iomanip.h>

#include "../../tk8.0.4/generic/tkCanvas.h"

#include "ps.h"
#include "frame.h"
#include "fitsimage.h"
#include "contour.h"
#include "util.h"
#include "nan.h"

// this is copied from tkCanvPs.c

typedef struct TkPostscriptInfo {
  int x, y, width, height;	/* Area to print, in canvas pixel
				 * coordinates. */
  int x2, y2;			/* x+width and y+height. */
  char *pageXString;		/* String value of "-pagex" option or NULL. */
  char *pageYString;		/* String value of "-pagey" option or NULL. */
  double pageX, pageY;  	/* Postscript coordinates (in points)
			         * corresponding to pageXString and
			         * pageYString. Don't forget that y-values
			         * grow upwards for Postscript! */
  char *pageWidthString;	/* Printed width of output. */
  char *pageHeightString;	/* Printed height of output. */
  double scale;		        /* Scale factor for conversion: each pixel
			         * maps into this many points. */
  Tk_Anchor pageAnchor;	        /* How to anchor bbox on Postscript page. */
  int rotate;			/* Non-zero means output should be rotated
				 * on page (landscape mode). */
  char *fontVar;		/* If non-NULL, gives name of global variable
				 * containing font mapping information.
				 * Malloc'ed. */
  char *colorVar;		/* If non-NULL, give name of global variable
				 * containing color mapping information.
				 * Malloc'ed. */
  char *colorMode;		/* Mode for handling colors:  "monochrome",
				 * "gray", or "color".  Malloc'ed. */
  int colorLevel;		/* Numeric value corresponding to colorMode:
				 * 0 for mono, 1 for gray, 2 for color. */
  char *fileName;		/* Name of file in which to write Postscript;
				 * NULL means return Postscript info as
				 * result. Malloc'ed. */
  char *channelName;		/* If -channel is specified, the name of
                                 * the channel to use. */
  Tcl_Channel chan;		/* Open channel corresponding to fileName. */
  Tcl_HashTable fontTable;	/* Hash table containing names of all font
				 * families used in output.  The hash table
				 * values are not used. */
  int prepass;		        /* Non-zero means that we're currently in
			         * the pre-pass that collects font information,
			         * so the Postscript generated isn't
			         * relevant. */
} TkPostscriptInfo;

int Frame::postscriptProc(int prepass)
{
  TkPostscriptInfo* psInfo = ((TkCanvas*)canvas)->psInfoPtr;

  if (!visible)
    return TCL_OK;

  if (prepass)
    return TCL_OK;

  char buf[256];
  Vector org = psOrigin();

  // Image

  if (fits) {

    // save graphics state

    Tcl_AppendResult(interp, "gsave\n", NULL);

    float scale = psInfo->scale * psResolution / 72.;
    int width = (int)(options->width * scale);
    int height = (int)(options->height * scale);

    // scale

    {
      ostrstream str(buf,256);
      str << org[0] << ' ' << org[1] << " translate" << endl
	  << 1/scale << ' ' << 1/scale << " scale" << endl
	  << ends;
      Tcl_AppendResult(interp, buf, NULL);
    }

    switch (psLevel) {
    case 1:
      switch (psColorSpace) {
      case BW:
      case GRAY:
	psLevel1(GRAY, width, height, scale);
	break;
      case RGB:
      case CMYK:
	psLevel1(RGB, width, height, scale);
	break;
      }
      break;
    case 2:
      psLevel2(psColorSpace, width, height, scale);
      break;
    }

    // restore gstate

    Tcl_AppendResult(interp, "grestore\n", NULL);
  }

  // Markers & Contours

  // clip path
  // I don't know why the height needs to be reduced by one, however, it works

  {
    ostrstream str(buf,256);
    str << org[0] << ' ' << org[1] << ' ' 
	<< options->width << ' ' << options->height-1
	<< " rectclip" << endl << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  switch (psLevel) {
  case 1:
    switch (psColorSpace) {
    case BW:
    case GRAY:
      psMarkers(GRAY);
      if (contour)
	contour->ps(GRAY);
      break;
    case RGB:
    case CMYK:
      psMarkers(RGB);
      if (contour)
	contour->ps(RGB);
      break;
    }
    break;
  case 2:
    psMarkers(psColorSpace);
    if (contour)
      contour->ps(psColorSpace);
    break;
  }

  return TCL_OK;
}

void Frame::psLevel1(PSColorSpace mode, int width, int height, float scale)
{
  if (fitsCount == 1)
    psLevel1Single(mode, width, height, scale);
  else
    psLevel1Mosaic(mode, width, height, scale);
}

void Frame::psLevel1Head(PSColorSpace mode, int width, int height)
{
  char buf[512];
  ostrstream str(buf,512);

  switch (mode) {
  case BW:
  case GRAY:
    str << "/picstr " << dec << width << " string def" << endl
	<< width << ' ' << height << " 8 matrix" << endl
	<< "{currentfile picstr readhexstring pop}" << endl
	<< "image" << endl;
    break;
  case RGB:
  case CMYK:
    str << "/picstr " << dec << width*3 << " string def" << endl
	<< width << ' ' << height << " 8 matrix" << endl
	<< "{currentfile picstr readhexstring pop}" << endl
	<< "false 3 colorimage" << endl;
    break;
  }

  str << ends;

  Tcl_AppendResult(interp, buf, NULL);
}

void Frame::psLevel1Single(PSColorSpace mode, int width, int height, 
			   float scale)
{
  psLevel1Head(mode, width, height);
  NoCompressAsciiHex filter;
  
  // size = 2 char per byte * 3 bytes per pixel + estimate of endls + 
  // endl + ends

  int size = (int)(width*3*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];

  int srcWidth = fits->width();
  int srcHeight = fits->height();

  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->psColors();

  Matrix m = (fits->getDataToRef() * psMatrix(scale, width, height)).invert();
  double ll = fits->getLowDouble();
  double hh = fits->getHighDouble();
  double diff = hh - ll;

  for (int j=height-1; j>=0; j--) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
      unsigned char red = 255;
      unsigned char green = 255;
      unsigned char blue = 255;
	    
      Vector s = Vector(i,j) * m;
      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = fits->getValueDouble(((int)s[1])*srcWidth + 
						     (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll) {
	    blue = table[0];
	    green = table[1];
	    red = table[2];
	  }
	  else if (value >= hh) {
	    blue = table[length*3];
	    green = table[length*3+1];
	    red = table[length*3+2];
	  }
	  else {
	    int l = (int)(((value - ll)/diff * length) + .5);
	    blue = table[l*3];
	    green = table[l*3+1];
	    red = table[l*3+2];
	  }
      }

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
      case CMYK:
	filter << red << green << blue;
	break;
      }
      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

void Frame::psLevel1Mosaic(PSColorSpace mode, int width, int height, 
			   float scale)
{
  psLevel1Head(mode, width, height);

  NoCompressAsciiHex filter;

  // size = 2 char per byte * 3 bytes per pixel + estimate of endls + 
  // endl + ends

  int size = (int)(width*3*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];

  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->psColors();

  Matrix m;
  int srcWidth;
  int srcHeight;

  double ll;
  double hh;
  double diff;

  int found = 0;
  FitsImage* current = fits;
  FitsImage* ptr;
  Matrix m1 = psMatrix(scale, width, height);

  for (int j=height-1; j>=0; j--) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
      unsigned char red = 255;
      unsigned char green = 255;
      unsigned char blue = 255;
	    
    tryagain:
      if (!found) {
	ptr = current;
	current = current->next();

	m = (ptr->getDataToRef() * m1).invert();

	srcWidth = ptr->width();
	srcHeight = ptr->height();

	ll = ptr->getLowDouble();
	hh = ptr->getHighDouble();
	diff = hh - ll;
      }

      Vector s = Vector(i,j) * m;

      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = ptr->getValueDouble(((int)s[1])*srcWidth + 
						    (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll) {
	    blue = table[0];
	    green = table[1];
	    red = table[2];
	  }
	  else if (value >= hh) {
	    blue = table[length*3];
	    green = table[length*3+1];
	    red = table[length*3+2];
	  }
	  else {
	    int l = (int)(((value - ll)/diff * length) + .5);
	    blue = table[l*3];
	    green = table[l*3+1];
	    red = table[l*3+2];
	  }
	found = 1;
      }
      else {
	found = 0;
	if (current)
	  goto tryagain;
	else
	  current = fits;
      }

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
      case CMYK:
	filter << red << green << blue;
	break;
      }
      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

void Frame::psLevel2(PSColorSpace mode, int width, int height, float scale)
{
  if (colorScale->size() < 256) {
    if (fitsCount == 1)
      psLevel2SingleIndex(mode, width, height, scale);
    else
      psLevel2MosaicIndex(mode, width, height, scale);
  }
  else {
    if (fitsCount == 1)
      psLevel2Single(mode, width, height, scale);
    else
      psLevel2Mosaic(mode, width, height, scale);
  }
}

void Frame::psLevel2Head(PSColorSpace mode, int width, int height)
{
  char buf[4096];
  ostrstream str(buf,4096);

  switch (mode) {
  case BW:
  case GRAY:
    str << "/DeviceGray setcolorspace" << endl;
    break;
  case RGB:
    str << "/DeviceRGB setcolorspace" << endl;
    break;
  case CMYK:
    str << "/DeviceCMYK setcolorspace" << endl;
    break;
  }

  str << "<<" << endl
      << "/ImageType 1" << endl
      << "/Width " << dec << width << endl
      << "/Height " << dec << height << endl
      << "/BitsPerComponent 8" << endl;

  switch (mode) {
  case BW:
  case GRAY:
    str << "/Decode [0 1]" << endl;
    break;
  case RGB:
    str << "/Decode [0 1 0 1 0 1]" << endl;
    break;
  case CMYK:
    str << "/Decode [0 1 0 1 0 1 0 1]" << endl;
    break;
  }

  if (psInterpolate)
    str << "/Interpolate true" << endl;

  str << "/ImageMatrix matrix" << endl
      << "/DataSource currentfile" << endl
      << "/ASCII85Decode filter" << endl
      << "/RunLengthDecode filter" << endl
      << ">>" << endl
      << "image" << endl
      << ends;

  Tcl_AppendResult(interp, buf, NULL);
}

void Frame::psLevel2Single(PSColorSpace mode, int width, int height, 
			   float scale)
{
  psLevel2Head(mode, width, height);
  RLEAscii85 filter;

  // size = 2 char per byte * 4 chars per pixel + estimate of endls + 
  // endl + ends

  int size = (int)(width*4*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];
	    
  int srcWidth = fits->width();
  int srcHeight = fits->height();

  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->psColors();

  Matrix m = (fits->getDataToRef() * psMatrix(scale, width, height)).invert();
  double ll = fits->getLowDouble();
  double hh = fits->getHighDouble();
  double diff = hh - ll;

  for (int j=height-1; j>=0; j--) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
      unsigned char red = 255;
      unsigned char green = 255;
      unsigned char blue = 255;
	    
      Vector s = Vector(i,j) * m;
      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = fits->getValueDouble(((int)s[1])*srcWidth + 
						     (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll) {
	    blue = table[0];
	    green = table[1];
	    red = table[2];
	  }
	  else if (value >= hh) {
	    blue = table[length*3];
	    green = table[length*3+1];
	    red = table[length*3+2];
	  }
	  else {
	    int l = (int)(((value - ll)/diff * length) + .5);
	    blue = table[l*3];
	    green = table[l*3+1];
	    red = table[l*3+2];
	  }
      }

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
	filter << red << green << blue;
	break;
      case CMYK:
	{
	  unsigned char cyan, magenta, yellow, black;
	  RGB2CMYK(red, green, blue, &cyan, &magenta, &yellow, &black);
	  filter << cyan << magenta << yellow << black;
	}
	break;
      }
      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << "~>" << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

void Frame::psLevel2Mosaic(PSColorSpace mode, int width, int height, 
			   float scale)
{
  psLevel2Head(mode, width, height);
  RLEAscii85 filter;

  // size = 2 char per byte * 4 bytes per pixel + estimate of endls + 
  // endl + ends

  int size = (int)(width*4*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];

  int length = colorScale->size() - 1;
  const unsigned char* table = colorScale->psColors();

  Matrix m;
  int srcWidth;
  int srcHeight;

  double ll;
  double hh;
  double diff;

  int found = 0;
  FitsImage* current = fits;
  FitsImage* ptr;
  Matrix m1 = psMatrix(scale, width, height);

  for (int j=height-1; j>=0; j--) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
      unsigned char red = 255;
      unsigned char green = 255;
      unsigned char blue = 255;

    tryagain:
      if (!found) {
	ptr = current;
	current = current->next();

	m = (ptr->getDataToRef() * m1).invert();

	srcWidth = ptr->width();
	srcHeight = ptr->height();

	ll = ptr->getLowDouble();
	hh = ptr->getHighDouble();
	diff = hh - ll;
      }

      Vector s = Vector(i,j) * m;

      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = ptr->getValueDouble(((int)s[1])*srcWidth + 
						    (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll) {
	    blue = table[0];
	    green = table[1];
	    red = table[2];
	  }
	  else if (value >= hh) {
	    blue = table[length*3];
	    green = table[length*3+1];
	    red = table[length*3+2];
	  }
	  else {
	    int l = (int)(((value - ll)/diff * length) + .5);
	    blue = table[l*3];
	    green = table[l*3+1];
	    red = table[l*3+2];
	  }
	found = 1;
      }
      else {
	found = 0;
	if (current)
	  goto tryagain;
	else
	  current = fits;
      }

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
	filter << red << green << blue;
	break;
      case CMYK:
	{
	  unsigned char cyan, magenta, yellow, black;
	  RGB2CMYK(red, green, blue, &cyan, &magenta, &yellow, &black);
	  filter << cyan << magenta << yellow << black;
	}
	break;
      }
      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << "~>" << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

void Frame::psLevel2HeadIndex(PSColorSpace mode, int width, int height)
{
  char buf[4096];
  ostrstream str(buf,4096);
  const unsigned short white = 255;

  switch (mode) {
  case BW:
  case GRAY:
    {
      str << "[/Indexed /DeviceGray 255" << endl 
	  << '<' << endl;

      int count = 0;
      for (int i=0; i<255; i++) {
	unsigned short value = 0;

	for (int j=0; j<colorCount; j++)
	  if (indexCells[j]==i) {
	    value = RGB2Gray(colorCells[j*3+2], colorCells[j*3+1], 
			     colorCells[j*3]);
	    break;
	  }

	str << hex << setfill('0') << setw(2) << value << ' ';

	if (++count == 16) {
	  str << endl;
	  count = 0;
	}
      }

      // set last location to white
      str << hex << setfill('0') << setw(2) << white << endl;

    }
    break;
  case RGB:
    {
      str << "[/Indexed /DeviceRGB 255" << endl 
	  << '<' << endl;

      int count = 0;
      for (int i=0; i<255; i++) {
	unsigned short red = 0;
	unsigned short green = 0;
	unsigned short blue = 0;

	for (int j=0; j<colorCount; j++)
	  if (indexCells[j]==i) {
	    red = colorCells[j*3+2];
	    green = colorCells[j*3+1];
	    blue = colorCells[j*3];
	    break;
	  }

	str << hex << setfill('0') << setw(2) << red 
	    << hex << setfill('0') << setw(2) << green
	    << hex << setfill('0') << setw(2) << blue
	    << ' ';

	if (++count == 8) {
	  str << endl;
	  count = 0;
	}
      }

      // set last location to white
      str << hex << setfill('0') << setw(2) << white
	  << hex << setfill('0') << setw(2) << white
	  << hex << setfill('0') << setw(2) << white
	  << ' ' << endl;

    }
    break;
  case CMYK:
    {
      str << "[/Indexed /DeviceCMYK 255" << endl 
	  << '<' << endl;

      int count = 0;
      for (int i=0; i<255; i++) {
	unsigned char c = 0;
	unsigned char m = 0;
	unsigned char y = 0;
	unsigned char b = 255;

	for (int j=0; j<colorCount; j++)
	  if (indexCells[j]==i) {
	    RGB2CMYK(colorCells[j*3+2], colorCells[j*3+1], colorCells[j*3],
		     &c, &m, &y, &b);
	    break;
	  }

	unsigned short cyan = c;
	unsigned short magenta = m;
	unsigned short yellow = y;
	unsigned short black = b;

	str << hex << setfill('0') << setw(2) << cyan 
	    << hex << setfill('0') << setw(2) << magenta
	    << hex << setfill('0') << setw(2) << yellow
	    << hex << setfill('0') << setw(2) << black
	    << ' ';

	if (++count == 8) {
	  str << endl;
	  count = 0;
	}
      }

      // set last location to white
      str << hex << setfill('0') << setw(2) << 0
	  << hex << setfill('0') << setw(2) << 0
	  << hex << setfill('0') << setw(2) << 0
	  << hex << setfill('0') << setw(2) << 0
	  << ' ' << endl;
    }
    break;
  }

  str << '>' << endl
      << "] setcolorspace" << endl
      << "<<" << endl
      << "/ImageType 1" << endl
      << "/Width " << dec << width << endl
      << "/Height " << dec << height << endl
      << "/BitsPerComponent 8" << endl
      << "/Decode [0 255]" << endl;

  if (psInterpolate)
    str << "/Interpolate true" << endl;

  str << "/ImageMatrix matrix" << endl
      << "/DataSource currentfile" << endl
      << "/ASCII85Decode filter" << endl
      << "/RunLengthDecode filter" << endl
      << ">>" << endl
      << "image" << endl
      << ends;

  Tcl_AppendResult(interp, buf, NULL);
}

void Frame::psLevel2SingleIndex(PSColorSpace mode, int width, int height, 
				float scale)
{
  psLevel2HeadIndex(mode, width, height);
  RLEAscii85 filter;

  // size = 2 char per byte + estimate of endls + endl + ends

  int size = (int)(width*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];
	    
  int srcWidth = fits->width();
  int srcHeight = fits->height();

  int length = colorScale->size() - 1;
  const unsigned short* table = colorScale->psIndex();

  Matrix m = (fits->getDataToRef() * psMatrix(scale, width, height)).invert();
  double ll = fits->getLowDouble();
  double hh = fits->getHighDouble();
  double diff = hh - ll;

  for (int j=height-1; j>=0; j--) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
      Vector s = Vector(i,j) * m;
      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = fits->getValueDouble(((int)s[1])*srcWidth + 
						     (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll)
	    filter << table[0];
	  else if (value >= hh)
	    filter << table[length];
	  else
	    filter << table[(int)(((value - ll)/diff * length) + .5)];
	else
	  filter << 255;
      }
      else
	filter << 255;

      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << "~>" << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

void Frame::psLevel2MosaicIndex(PSColorSpace mode, int width, int height, 
				float scale)
{
  psLevel2HeadIndex(mode, width, height);
  RLEAscii85 filter;

  // size = 2 char per byte * 3 bytes per pixel + estimate of endls + 
  // endl + ends

  int size = (int)(width*3*2*(1 + 1./80.) +.5) + 2;
  char* buf = new char[size];

  int length = colorScale->size() - 1;
  const unsigned short* table = colorScale->psIndex();

  Matrix m;
  int srcWidth;
  int srcHeight;

  double ll;
  double hh;
  double diff;

  int found = 0;
  FitsImage* current = fits;
  FitsImage* ptr;
  Matrix m1 = psMatrix(scale, width, height);

  for (int j=height-1; j>=0; j--) {
    ostrstream str(buf,size);

    for (int i=0; i<width; i++) {
    tryagain:
      if (!found) {
	ptr = current;
	current = current->next();

	m = (ptr->getDataToRef() * m1).invert();

	srcWidth = ptr->width();
	srcHeight = ptr->height();

	ll = ptr->getLowDouble();
	hh = ptr->getHighDouble();
	diff = hh - ll;
      }

      Vector s = Vector(i,j) * m;

      if (s[0] >= 0 && s[0] < srcWidth && s[1] >= 0 && s[1] < srcHeight) {
	register double value = ptr->getValueDouble(((int)s[1])*srcWidth + 
						    (int)s[0]);
	
	if (!isnand(value))
	  if (value <= ll)
	    filter << table[0];
	  else if (value >= hh)
	    filter << table[length];
	  else
	    filter << table[(int)(((value - ll)/diff * length) + .5)];

	found = 1;
      }
      else {
	found = 0;
	if (current)
	  goto tryagain;
	else {
	  filter << 255;
	  current = fits;
	}
      }

      str << filter;
    }
    str << ends;
    Tcl_AppendResult(interp, buf, NULL);
  }

  ostrstream str(buf,size);
  filter.flush(str);
  str << endl << "~>" << endl << ends;
  Tcl_AppendResult(interp, buf, NULL);
  delete buf;
}

Matrix Frame::psMatrix(float psZoom, int width, int height)
{
  Vector center = getCenter();
  return 
    Translate(-center) *
    FlipY() *
    Translate(center) *

    Translate(-cursor) *                // translate to cursor position
    Rotate(rotation) *                  // rotate about cursor position
    Rotate(wcsRotation) *               // rotate about center position
    orientationMatrix *                 // flip x/y axis about cursor position
    wcsOrientationMatrix *              // flip x/y axis about center
    Scale(zoom) *                       // scale about cursor position
    Scale(wcsZoom) *                    // scale about wcs scale
    Scale(psZoom) *

    Translate(Vector(width,height)/2.);
}

Vector Frame::psOrigin()
{
  // compute coordinates of lower-left corner of the image
  // taking into account the anchor position for the image.
  // this mapping is different than in widget.c, due to postscript origin 
  // (y is flipped)

  double xx = options->x;
  double yy = Tk_CanvasPsY(canvas, options->y);
     
  switch (options->anchor) {
  case TK_ANCHOR_NW:
    yy -= options->height;
    break;
  case TK_ANCHOR_N:
    xx -= options->width/2.0;
    yy -= options->height;
    break;
  case TK_ANCHOR_NE:
    xx -= options->width;
    yy -= options->height;
    break;
  case TK_ANCHOR_E:
    xx -= options->width;
    yy -= options->height/2.0;
    break;
  case TK_ANCHOR_SE:
    xx -= options->width;
    break;
  case TK_ANCHOR_S:
    xx -= options->width/2.0;
    break;
  case TK_ANCHOR_SW:
    break;
  case TK_ANCHOR_W:
    yy -= options->height/2.0;
    break;
  case TK_ANCHOR_CENTER:
    xx -= options->width/2.0;
    yy -= options->height/2.0;
    break;
  }

  return Vector(xx,yy);
}

extern "C" {

int TkGetProlog(Tcl_Interp *interp)
{
  char* prolog = 
{"\
%%BeginProlog\n\
50 dict begin\n\
\n\
% This is a standard prolog for Postscript generated by Tk's canvas\n\
% widget.\n\
% SCCS: @(#) prolog.ps 1.7 96/07/08 17:52:14\n\
\n\
% The definitions below just define all of the variables used in\n\
% any of the procedures here.  This is needed for obscure reasons\n\
% explained on p. 716 of the Postscript manual (Section H.2.7,\n\
% 'Initializing Variables', in the section on Encapsulated Postscript).\n\
\n\
/baseline 0 def\n\
/stipimage 0 def\n\
/height 0 def\n\
/justify 0 def\n\
/lineLength 0 def\n\
/spacing 0 def\n\
/stipple 0 def\n\
/strings 0 def\n\
/xoffset 0 def\n\
/yoffset 0 def\n\
/tmpstip null def\n\
\n\
% Define the array ISOLatin1Encoding (which specifies how characters are\n\
% encoded for ISO-8859-1 fonts), if it isn't already present (Postscript\n\
% level 2 is supposed to define it, but level 1 doesn't).\n\
\n\
systemdict /ISOLatin1Encoding known not {\n\
    /ISOLatin1Encoding [\n\
	/space /space /space /space /space /space /space /space\n\
	/space /space /space /space /space /space /space /space\n\
	/space /space /space /space /space /space /space /space\n\
	/space /space /space /space /space /space /space /space\n\
	/space /exclam /quotedbl /numbersign /dollar /percent /ampersand\n\
	    /quoteright\n\
	/parenleft /parenright /asterisk /plus /comma /minus /period /slash\n\
	/zero /one /two /three /four /five /six /seven\n\
	/eight /nine /colon /semicolon /less /equal /greater /question\n\
	/at /A /B /C /D /E /F /G\n\
	/H /I /J /K /L /M /N /O\n\
	/P /Q /R /S /T /U /V /W\n\
	/X /Y /Z /bracketleft /backslash /bracketright /asciicircum /underscore\n\
	/quoteleft /a /b /c /d /e /f /g\n\
	/h /i /j /k /l /m /n /o\n\
	/p /q /r /s /t /u /v /w\n\
	/x /y /z /braceleft /bar /braceright /asciitilde /space\n\
	/space /space /space /space /space /space /space /space\n\
	/space /space /space /space /space /space /space /space\n\
	/dotlessi /grave /acute /circumflex /tilde /macron /breve /dotaccent\n\
	/dieresis /space /ring /cedilla /space /hungarumlaut /ogonek /caron\n\
	/space /exclamdown /cent /sterling /currency /yen /brokenbar /section\n\
	/dieresis /copyright /ordfeminine /guillemotleft /logicalnot /hyphen\n\
	    /registered /macron\n\
	/degree /plusminus /twosuperior /threesuperior /acute /mu /paragraph\n\
	    /periodcentered\n\
	/cedillar /onesuperior /ordmasculine /guillemotright /onequarter\n\
	    /onehalf /threequarters /questiondown\n\
	/Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n\
	/Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex\n\
	    /Idieresis\n\
	/Eth /Ntilde /Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply\n\
	/Oslash /Ugrave /Uacute /Ucircumflex /Udieresis /Yacute /Thorn\n\
	    /germandbls\n\
	/agrave /aacute /acircumflex /atilde /adieresis /aring /ae /ccedilla\n\
	/egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex\n\
	    /idieresis\n\
	/eth /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide\n\
	/oslash /ugrave /uacute /ucircumflex /udieresis /yacute /thorn\n\
	    /ydieresis\n\
    ] def\n\
} if\n\
\n\
% font ISOEncode font\n\
% This procedure changes the encoding of a font from the default\n\
% Postscript encoding to ISOLatin1.  It's typically invoked just\n\
% before invoking 'setfont'.  The body of this procedure comes from\n\
% Section 5.6.1 of the Postscript book.\n\
\n\
/ISOEncode {\n\
    dup length dict begin\n\
	{1 index /FID ne {def} {pop pop} ifelse} forall\n\
	/Encoding ISOLatin1Encoding def\n\
	currentdict\n\
    end\n\
\n\
    % I'm not sure why it's necessary to use 'definefont' on this new\n\
    % font, but it seems to be important; just use the name 'Temporary'\n\
    % for the font.\n\
\n\
    /Temporary exch definefont\n\
} bind def\n\
\n\
% StrokeClip\n\
%\n\
% This procedure converts the current path into a clip area under\n\
% the assumption of stroking.  It's a bit tricky because some Postscript\n\
% interpreters get errors during strokepath for dashed lines.  If\n\
% this happens then turn off dashes and try again.\n\
\n\
/StrokeClip {\n\
    {strokepath} stopped {\n\
	(This Postscript printer gets limitcheck overflows when) =\n\
	(stippling dashed lines;  lines will be printed solid instead.) =\n\
	[] 0 setdash strokepath} if\n\
    clip\n\
} bind def\n\
\n\
% desiredSize EvenPixels closestSize\n\
%\n\
% The procedure below is used for stippling.  Given the optimal size\n\
% of a dot in a stipple pattern in the current user coordinate system,\n\
% compute the closest size that is an exact multiple of the device's\n\
% pixel size.  This allows stipple patterns to be displayed without\n\
% aliasing effects.\n\
\n\
/EvenPixels {\n\
    % Compute exact number of device pixels per stipple dot.\n\
    dup 0 matrix currentmatrix dtransform\n\
    dup mul exch dup mul add sqrt\n\
\n\
    % Round to an integer, make sure the number is at least 1, and compute\n\
    % user coord distance corresponding to this.\n\
    dup round dup 1 lt {pop 1} if\n\
    exch div mul\n\
} bind def\n\
\n\
% width height string StippleFill --\n\
%\n\
% Given a path already set up and a clipping region generated from\n\
% it, this procedure will fill the clipping region with a stipple\n\
% pattern.  'String' contains a proper image description of the\n\
% stipple pattern and 'width' and 'height' give its dimensions.  Each\n\
% stipple dot is assumed to be about one unit across in the current\n\
% user coordinate system.  This procedure trashes the graphics state.\n\
\n\
/StippleFill {\n\
    % The following code is needed to work around a NeWSprint bug.\n\
\n\
    /tmpstip 1 index def\n\
\n\
    % Change the scaling so that one user unit in user coordinates\n\
    % corresponds to the size of one stipple dot.\n\
    1 EvenPixels dup scale\n\
\n\
    % Compute the bounding box occupied by the path (which is now\n\
    % the clipping region), and round the lower coordinates down\n\
    % to the nearest starting point for the stipple pattern.  Be\n\
    % careful about negative numbers, since the rounding works\n\
    % differently on them.\n\
\n\
    pathbbox\n\
    4 2 roll\n\
    5 index div dup 0 lt {1 sub} if cvi 5 index mul 4 1 roll\n\
    6 index div dup 0 lt {1 sub} if cvi 6 index mul 3 2 roll\n\
\n\
    % Stack now: width height string y1 y2 x1 x2\n\
    % Below is a doubly-nested for loop to iterate across this area\n\
    % in units of the stipple pattern size, going up columns then\n\
    % across rows, blasting out a stipple-pattern-sized rectangle at\n\
    % each position\n\
\n\
    6 index exch {\n\
	2 index 5 index 3 index {\n\
	    % Stack now: width height string y1 y2 x y\n\
\n\
	    gsave\n\
	    1 index exch translate\n\
	    5 index 5 index true matrix tmpstip imagemask\n\
	    grestore\n\
	} for\n\
	pop\n\
    } for\n\
    pop pop pop pop pop\n\
} bind def\n\
\n\
% -- AdjustColor --\n\
% Given a color value already set for output by the caller, adjusts\n\
% that value to a grayscale or mono value if requested by the CL\n\
% variable.\n\
\n\
/AdjustColor {\n\
    CL 2 lt {\n\
	currentgray\n\
	CL 0 eq {\n\
	    .5 lt {0} {1} ifelse\n\
	} if\n\
	setgray\n\
    } if\n\
} bind def\n\
\n\
% x y strings spacing xoffset yoffset justify stipple DrawText --\n\
% This procedure does all of the real work of drawing text.  The\n\
% color and font must already have been set by the caller, and the\n\
% following arguments must be on the stack:\n\
%\n\
% x, y -	Coordinates at which to draw text.\n\
% strings -	An array of strings, one for each line of the text item,\n\
%		in order from top to bottom.\n\
% spacing -	Spacing between lines.\n\
% xoffset -	Horizontal offset for text bbox relative to x and y: 0 for\n\
%		nw/w/sw anchor, -0.5 for n/center/s, and -1.0 for ne/e/se.\n\
% yoffset -	Vertical offset for text bbox relative to x and y: 0 for\n\
%		nw/n/ne anchor, +0.5 for w/center/e, and +1.0 for sw/s/se.\n\
% justify -	0 for left justification, 0.5 for center, 1 for right justify.\n\
% stipple -	Boolean value indicating whether or not text is to be\n\
%		drawn in stippled fashion.  If text is stippled,\n\
%		procedure StippleText must have been defined to call\n\
%		StippleFill in the right way.\n\
%\n\
% Also, when this procedure is invoked, the color and font must already\n\
% have been set for the text.\n\
\n\
/DrawText {\n\
    /stipple exch def\n\
    /justify exch def\n\
    /yoffset exch def\n\
    /xoffset exch def\n\
    /spacing exch def\n\
    /strings exch def\n\
\n\
    % First scan through all of the text to find the widest line.\n\
\n\
    /lineLength 0 def\n\
    strings {\n\
	stringwidth pop\n\
	dup lineLength gt {/lineLength exch def} {pop} ifelse\n\
	newpath\n\
    } forall\n\
\n\
    % Compute the baseline offset and the actual font height.\n\
\n\
    0 0 moveto (TXygqPZ) false charpath\n\
    pathbbox dup /baseline exch def\n\
    exch pop exch sub /height exch def pop\n\
    newpath\n\
\n\
    % Translate coordinates first so that the origin is at the upper-left\n\
    % corner of the text's bounding box. Remember that x and y for\n\
    % positioning are still on the stack.\n\
\n\
    translate\n\
    lineLength xoffset mul\n\
    strings length 1 sub spacing mul height add yoffset mul translate\n\
\n\
    % Now use the baseline and justification information to translate so\n\
    % that the origin is at the baseline and positioning point for the\n\
    % first line of text.\n\
\n\
    justify lineLength mul baseline neg translate\n\
\n\
    % Iterate over each of the lines to output it.  For each line,\n\
    % compute its width again so it can be properly justified, then\n\
    % display it.\n\
\n\
    strings {\n\
	dup stringwidth pop\n\
	justify neg mul 0 moveto\n\
	stipple {\n\
\n\
	    % The text is stippled, so turn it into a path and print\n\
	    % by calling StippledText, which in turn calls StippleFill.\n\
	    % Unfortunately, many Postscript interpreters will get\n\
	    % overflow errors if we try to do the whole string at\n\
	    % once, so do it a character at a time.\n\
\n\
	    gsave\n\
	    /char (X) def\n\
	    {\n\
		char 0 3 -1 roll put\n\
		currentpoint\n\
		gsave\n\
		char true charpath clip StippleText\n\
		grestore\n\
		char stringwidth translate\n\
		moveto\n\
	    } forall\n\
	    grestore\n\
	} {show} ifelse\n\
	0 spacing neg translate\n\
    } forall\n\
} bind def\n\
\n\
%%EndProlog\n\
"};

  Tcl_AppendResult(interp, prolog, NULL);
  return TCL_OK;
}

}
