#include "text.h"
#include "frame.h"
#include "util.h"

Text::Text(Frame* p, const Vector& ctr, const char* clr, const char* f, 
	   const char* t, unsigned short prop) 
  : Marker(p, ctr, clr, f, t, prop)
{
  strcpy(type,"text");
  handle = new Vector[4];
  numHandle = 4;

  updateBBox();
}

void Text::updateBBox()
{
  Vector c = center * parent->refToCanvas;
  if (text && font) {
    Tk_FontMetrics metrics;
    Tk_GetFontMetrics(font, &metrics);
    int width = Tk_TextWidth(font, text, strlen(text));

    bbox = BBox(c * 
		Translate(-width/2.,  (metrics.ascent+metrics.descent)/2.), 
		c * 
		Translate( width/2., -(metrics.ascent+metrics.descent)/2.));
  }
  else {
    bbox = BBox(c,c);
    bbox.expand(5);
  }

  // generate handles

  handle[0] = bbox.ll;
  handle[1] = bbox.lr();
  handle[2] = bbox.ur;
  handle[3] = bbox.ul();

  // make room for handles

  bbox.expand(3);
  allBBox = bbox;
}

void Text::draw()
{
  XSetForeground(display, gc, color);
  noDash(gc);

  drawHandles();

  if (font)
    XSetFont(display, gc, Tk_FontId(font));

  if (text && font) {
    Tk_FontMetrics metrics;
    Tk_GetFontMetrics(font, &metrics);
    int width = Tk_TextWidth(font, text, strlen(text));

    Vector t = (center*parent->refToWidget) * 
      Translate(-width/2., (metrics.ascent-metrics.descent)/2.);
    XDrawString(display, parent->pixmap, gc, (int)t[0], (int)t[1], 
		text, strlen(text));
  }
}

void Text::drawXOR()
{
  Vector b = bbox.ll * parent->canvasToWindow;
  Vector size = bbox.size() - Vector(1,1);
  XDrawRectangle(display, Tk_WindowId(parent->tkwin), gcxor,
		 (int)b[0], (int)b[1], (int)size[0], (int)size[1]);
}

void Text::drawMagnifier(const Matrix& m)
{
  XSetForeground(display, gc, color);
  noDash(gc);

  if (font)
    XSetFont(display, gc, Tk_FontId(font));

  if (text && font) {
    Tk_FontMetrics metrics;
    Tk_GetFontMetrics(font, &metrics);
    int width = Tk_TextWidth(font, text, strlen(text));

    Vector t = (center*m) * 
      Translate(-width/2., (metrics.ascent-metrics.descent)/2.);
    XDrawString(display, parent->magnifierPixmap, gc, 
		(int)t[0], (int)t[1], text, strlen(text));
  }
}

void Text::ps(int mode)
{
  psColor(mode);
  psNoDash();

  if (text && font) {
    Tcl_DString psFont;
    Tcl_DStringInit(&psFont);
    int psSize = Tk_PostscriptFontName(font, &psFont);

    Tk_FontMetrics metrics;
    Tk_GetFontMetrics(font, &metrics);

    char buf[512];
    ostrstream str(buf,512);
    str << '/' << Tcl_DStringValue(&psFont) 
	<< " findfont " << psSize << " scalefont setfont" << endl;
      
    Tcl_DStringFree(&psFont);

    str << (center * parent->refToCanvas).TkCanvasPs(parent->canvas) 
	<< " moveto" << endl
	<< '(' << text << ')' << endl
    	<< "dup stringwidth pop 2 div 0 exch sub "
	<< (int)(-(metrics.ascent-metrics.descent)/2) 
	<< " rmoveto show" << endl
	<< endl << ends;

    Tcl_AppendResult(parent->interp, buf, NULL);
  }
}

// list

void Text::listSAOtng(ostream& str, int sys, int format)
{
  if (properties&INCLUDE)
    str << '+';
  else
    str << '-';

  switch ((Frame::CoordSystem)sys) {
  case Frame::IMAGE:
  case Frame::DETECTOR:
  case Frame::AMPLIFIER:
    {
      Vector v = parent->map(center,Frame::REF,Frame::IMAGE);
      str << type << '(' << setprecision(8) << v[0] << ',' << v[1] 
	  << ", \"" << text << "\") ";
    }
    break;
  case Frame::PHYSICAL:
    {
      Vector v = parent->map(center,Frame::REF,Frame::PHYSICAL);
      str << type << '(' << setprecision(8) << v[0] << ',' << v[1] 
	  << ", \"" << text << "\") ";
    }
    break;

  case Frame::EQUATORIAL_FK4:
  case Frame::EQUATORIAL_FK5:
  case Frame::ECLIPTIC:
  case Frame::GALACTIC:
  case Frame::ALTAZ:
  case Frame::LINEAR:
  case Frame::ICRS:
    switch ((Frame::CoordFormat)format) {
    case Frame::DEGREES:
      {
	Vector v = parent->map(center,Frame::REF,(Frame::CoordSystem)sys);
	str << type << '(' << setprecision(8) << v[0] << ',' << v[1] << ", \""
	    << text << "\") ";
      }
      break;
    case Frame::HMS:
      {
	char buf[64];
	parent->mapToWCSStr(center, Frame::REF, (Frame::CoordSystem)sys,
			    (Frame::CoordFormat)format, buf, 64);
	char ra[16];
	char dec[16];
	istrstream wcs(buf,64);
	wcs >> ra >> dec;
	str << type << '(' << ra << ',' << dec
	    << ", \"" << text << "\") ";
      }
      break;
    }
    break;
  }

  str << "# " << colorName;
}

