#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <iostream.h>

#include "util.h"

const double D_PI = 3.14159265358979323846264338327950288419716939937511;
int hmsSign; // used by parser and lex to indicate sign of dms or hms

char* dupstr(const char* str)
{
  char* copy;
  if (str) {
    copy=new char[strlen(str)+1];
    strcpy(copy,str);
  }
  else
    copy=NULL;

  return copy;
}

char* toLower(const char* str)
{
  char* r = dupstr(str);
  char* ptr = r;
  while (*ptr) {
    *ptr = (char)(tolower(((int)(*ptr))));
    ptr++;
  }
  return r;
}

char* toUpper(const char* str)
{
  char* r = dupstr(str);
  char* ptr =r;
  while (*ptr) {
    *ptr = (char)(toupper(((int)(*ptr))));
    ptr++;
  }
  return r;
}

double degToRad(double d)
{
  return D_PI * d / 180.;
}

double radToDeg(double r)
{
  double d = 180. * r / D_PI;

  if (d > 0) 
    while(d >= 360)
      d -= 360.;
  else
    while(d < 0)
      d += 360.;

  return d;
}

double hmsToDegree(int sign, double hour, double min, double sec)
{
  double r = sign * ((hour/24.*360.) + (min/60./24.*360.) + (sec/60./60./24.*360.));
  return r;
}

double dmsToDegree(int sign, double degree, double min, double sec)
{
  double r = sign * (degree + (min/60.) + (sec/60./60.));
  return r;
}

double parseHMS(const char* h)
{
  char* hms = dupstr(h); // its going to get clobbered
  char* str = hms;
  int sign = 1;
  if (str[0]=='-') {
    sign = -1;
    str++;
  }  
  int hour = atoi(strtok(str,":"));
  int min = atoi(strtok(NULL,":"));
  float sec = atof(strtok(NULL,":"));

  delete hms;

  return hmsToDegree(sign,hour,min,sec);
}

double parseDMS(const char* d)
{
  char* dms = dupstr(d); // its going to get clobbered
  char* str = dms;
  int sign = 1;
  if (str[0]=='-') {
    sign = -1;
    str++;
  }  
  int degree = atoi(strtok(str,":"));
  int min = atoi(strtok(NULL,":"));
  float sec = atof(strtok(NULL,":"));

  delete dms;

  return dmsToDegree(sign,degree,min,sec);
}

unsigned char decodeMask(unsigned char mask, int* s)
{
  *s=0;
  for (int i=0; i<8; i++, (*s)++) {
    if (mask & 0x80)
      break;
    mask <<= 1;
  }
  return mask;
}

unsigned short decodeMask(unsigned short mask, int* s)
{
  // first calc ffs for mask

  unsigned short m1 = mask;
  int m1s;
  for (m1s=0; m1s<16; m1s++) {
    if (m1 & 0x1)
      break;
    m1 >>= 1;
  }

  // then shift mask to mask a char

  for (int i=0; i<16; i++) {
    if (mask & 0x8000)
      break;
    mask <<= 1;
  }
  mask >>= 8;

  // now calc ffs for new mask

  unsigned short m2 = mask;
  int m2s;
  for (m2s=0; m2s<16; m2s++) {
    if (m2 & 0x1)
      break;
    m2 >>= 1;
  }

  *s = m1s-m2s;
  return mask;
}

int decodeMask(unsigned long mask)
{
  switch (mask) {
  case 0xff:
    return 0;
  case 0xff00:
    return 8;
  case 0xff0000:
    return 16;
  case 0xff000000:
    return 24;
  }
}

double RGB2Gray(double red, double green, double blue)
{
  return 0.30*red + 0.59*green + 0.11*blue;
}

unsigned char RGB2Gray(unsigned char red, unsigned char green, unsigned char blue)
{
  return (unsigned char)(0.30*red + 0.59*green + 0.11*blue);
}

void RGB2CMYK(unsigned char red, unsigned char green, unsigned char blue,
	       unsigned char* cyan, unsigned char* magenta, unsigned char* yellow,
	       unsigned char* black)
{
  // convert To CMY

  *cyan = 255-red;
  *magenta = 255-green;
  *yellow = 255-blue;
  *black =0;

  // determine black

  *black = 255;
  if (*cyan < *black)
    *black = *cyan;
  if (*magenta < *black)
    *black = *magenta;
  if (*yellow < *black)
    *black = *yellow;

  // substract out black

  *cyan -= *black;
  *magenta -= *black;
  *yellow -= *black;
}

ostream& psColorGray(const char* colorName, ostream& str)
{
  str << dec; // we want decimal
  if (!strcmp(colorName,"black"))
    str << "0.0";
  else if (!strcmp(colorName,"white"))
    str << "1.0";
  else if (!strcmp(colorName,"red"))
    str << RGB2Gray(1.,0.,0.);
  else if (!strcmp(colorName,"green"))
    str << RGB2Gray(0.,1.,0.);
  else if (!strcmp(colorName,"blue"))
    str << RGB2Gray(0.,0.,1.);
  else if (!strcmp(colorName,"cyan"))
    str << RGB2Gray(0.,1.,1.);
  else if (!strcmp(colorName,"magenta"))
    str << RGB2Gray(1.,0.,1.);
  else if (!strcmp(colorName,"yellow"))
    str << RGB2Gray(1.,1.,0.);
  else {
    cerr << "psColorGray Internal Error: Unable to Parser color: "<< colorName << endl;
    str << "0.0";
  }

  return str;
}

ostream& psColorRGB(const char* colorName, ostream& str)
{
  str << dec; // we want decimal
  if (!strcmp(colorName,"black"))
    str << "0 0 0";
  else if (!strcmp(colorName,"white"))
    str << "1 1 1";
  else if (!strcmp(colorName,"red"))
    str << "1 0 0";
  else if (!strcmp(colorName,"green"))
    str << "0 1 0";
  else if (!strcmp(colorName,"blue"))
    str << "0 0 1";
  else if (!strcmp(colorName,"cyan"))
    str << "0 1 1";
  else if (!strcmp(colorName,"magenta"))
    str << "1 0 1";
  else if (!strcmp(colorName,"yellow"))
    str << "1 1 0";
  else {
    cerr << "psColorRGB Internal Error: Unable to Parser color: "<< colorName << endl;
    str << "0 0 0";
  }

  return str;
}

ostream& psColorCMYK(const char* colorName, ostream& str)
{
  str << dec; // we want decimal
  if (!strcmp(colorName,"black"))
    str << "0 0 0 1";
  else if (!strcmp(colorName,"white"))
    str << "0 0 0 0";
  else if (!strcmp(colorName,"red"))
    str << "0 1 1 0";
  else if (!strcmp(colorName,"green"))
    str << "1 0 1 0";
  else if (!strcmp(colorName,"blue"))
    str << "1 1 0 0";
  else if (!strcmp(colorName,"cyan"))
    str << "1 0 0 0";
  else if (!strcmp(colorName,"magenta"))
    str << "0 1 0 0";
  else if (!strcmp(colorName,"yellow"))
    str << "0 0 1 0";
  else {
    cerr << "psColorCMYK Internal Error: Unable to Parser color: "<< colorName << endl;
    str << "0 0 0 1";
  }

  return str;
}

